-- lua/gemini/api.lua

local M = {}

-- Store conversation history
local conversation_history = {}

-- Helper function to get API key
local function get_api_key()
	return vim.g.gemini_api_key or os.getenv("GEMINI_API_KEY")
end

-- Async HTTP request function
local function async_request(url, payload, callback)
	-- Parse URL properly
	local host = url:match("^https://([^/]+)")
	local path = url:match("^https://[^/]+(.*)$")
	
	if not host or not path then
		vim.schedule(function()
			callback(nil, "Invalid URL format")
		end)
		return
	end

	local client = vim.loop.new_tcp()
	
	client:connect(host, 443, function(err)
		if err then 
			vim.schedule(function()
				callback(nil, "Connection error: " .. err)
			end)
			return
		end

		local ssl = vim.loop.new_tls()
		ssl:wrap(client)
		
		local request = string.format(
			"POST %s HTTP/1.1\r\n" ..
			"Host: %s\r\n" ..
			"Content-Type: application/json\r\n" ..
			"Content-Length: %d\r\n" ..
			"\r\n" ..
			"%s",
			path, host, #payload, payload
		)
		
		local response = ""
		
		ssl:write(request, function(err)
			if err then
				vim.schedule(function()
					callback(nil, "Write error: " .. err)
				end)
				return
			end
			
			ssl:read_start(function(err, chunk)
				if err then
					vim.schedule(function()
						callback(nil, "Read error: " .. err)
					end)
					return
				end
				
				if chunk then
					response = response .. chunk
				else
					-- Connection closed, process response
					local body = response:match("\r\n\r\n(.+)$")
					local success, decoded = pcall(vim.json.decode, body)
					
					vim.schedule(function()
						if success then
							callback(decoded)
						else
							callback(nil, "JSON decode error: " .. body)
						end
					end)
					
					ssl:close()
					client:close()
				end
			end)
		end)
	end)
end

function M.get_response(prompt, context, callback)
	local api_key = get_api_key()
	
	if not api_key then
		vim.schedule(function()
			callback(nil, "API key not set")
		end)
		return
	end

	local model = "gemini-2.0-flash"
	local contents = {}
	
	-- Add conversation history
	for _, message in ipairs(conversation_history) do
		table.insert(contents, {
			role = message.role,
			parts = {{
				text = message.content
			}}
		})
	end

	-- Add current prompt
	if context then
		table.insert(contents, {
			role = "user",
			parts = {{
				text = "Context:\n" .. context .. "\n\nQuery: " .. prompt
			}}
		})
	else
		table.insert(contents, {
			role = "user",
			parts = {{
				text = prompt
			}}
		})
	end

	-- Store prompt in history
	table.insert(conversation_history, {
		role = "user",
		content = prompt
	})

	local payload = vim.json.encode({
		contents = contents,
	})

	local url = string.format(
		"https://generativelanguage.googleapis.com/v1/models/%s:generateContent?key=%s",
		model,
		api_key
	)

	async_request(url, payload, function(result, error)
		if error then
			callback(nil, error)
			return
		end

		if result.error then
			callback(nil, "API Error: " .. vim.inspect(result.error))
			return
		end

		if result.candidates and
		   result.candidates[1] and
		   result.candidates[1].content and
		   result.candidates[1].content.parts and
		   result.candidates[1].content.parts[1] and
		   result.candidates[1].content.parts[1].text then
			
			local response_text = result.candidates[1].content.parts[1].text
			
			-- Store response in history
			table.insert(conversation_history, {
				role = "model",
				content = response_text
			})
			
			callback(response_text)
		else
			callback(nil, "Unexpected response structure")
		end
	end)
end

-- Add function to clear conversation history
function M.clear_conversation()
	conversation_history = {}
end

return M
